﻿using ELTE.TravelAgency.Models;
using System;
using System.Linq;
using System.Web.Mvc;

namespace ELTE.TravelAgency.Controllers
{
    /// <summary>
    /// Foglalások vezérlője.
    /// </summary>
    public class RentController : Controller
    {
        private TravelAgencyEntities _entities;

        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        public RentController() 
        {
            _entities = new TravelAgencyEntities();

            ViewBag.Cities = _entities.City.ToArray(); // a városokat berakjuk egy tömbbe
        }

        /// <summary>
        /// Foglalás (oldal lekérése).
        /// </summary>
        /// <param name="apartmentId"></param>
        /// <returns></returns>
        [HttpGet]
        public ActionResult Index(Int32? apartmentId)
        {
            if (apartmentId == null) // nem adtak meg azonosítót
                return RedirectToAction("Index", "Home"); // átirányítjuk a kezdőoldalra

            Apartment apartment = _entities.Apartment.Include("Building").Include("Building.City").FirstOrDefault(ap => ap.Id == apartmentId);

            if (apartment == null)
                return RedirectToAction("Index", "Home");

            RentViewModel rent = new RentViewModel { Apartment = apartment }; // létrehozunk egy új foglalást, amelynek megadjuk az apartmant

            // beállítunk egy foglalást, amely a következő megfelelő fordulónappal (minimum 1 héttel később), és egy hetes időtartammal rendelkezik
            rent.RentStartDate = DateTime.Today + TimeSpan.FromDays(7);
            while (rent.RentStartDate.DayOfWeek != apartment.TurndayOfWeek)
                rent.RentStartDate += TimeSpan.FromDays(1);

            rent.RentEndDate = rent.RentStartDate + TimeSpan.FromDays(7);

            return View("Index", rent); // visszaadjuk a foglalási nézetet, üres modellel
        }

        /// <summary>
        /// Foglalás (adatok beküldése).
        /// </summary>
        /// <param name="apartmentId">Apartman azonosítója.</param>
        /// <param name="rent">Foglalás adatai.</param>
        /// <returns>Foglalás eredmény nézete.</returns>
        [HttpPost]
        [ValidateAntiForgeryToken] // védelem XSRF támadás ellen
        public ActionResult Index(Int32? apartmentId, RentViewModel rent)
        {
            if (apartmentId == null || rent == null)
                return RedirectToAction("Index", "Home");

            // mivel a csatolt apartman adatok elvesznek, újra be kell tölteni
            Apartment apartment = _entities.Apartment.Include("Building").Include("Building.City").FirstOrDefault(ap => ap.Id == apartmentId);

            if (apartment == null)
                return RedirectToAction("Index", "Home");

            rent.Apartment = apartment;

            // végrehajtjuk az ellenőrzéseket            
            if (rent.RentStartDate < DateTime.Now + TimeSpan.FromDays(7) || rent.RentStartDate.DayOfWeek != rent.Apartment.TurndayOfWeek)
                ModelState.AddModelError("RentStartDate", "A kezdés dátuma nem megfelelő (túl korai, vagy nem fordulónapra esik)!");

            if (rent.RentEndDate < rent.RentStartDate)
                ModelState.AddModelError("RentEndDate", "A megadott foglalási idő érvénytelen (a foglalás vége korábban van, mint a kezdete)!");

            if (Convert.ToInt32((rent.RentEndDate - rent.RentEndDate).TotalDays) % 7 != 0 || rent.RentEndDate < rent.RentStartDate + TimeSpan.FromDays(7))
                ModelState.AddModelError("RentEndDate", "A megadott foglalási idő érvénytelen (egész heteket lehet csak foglalni)!");

            if (_entities.Rent.Where(r => r.ApartmentId == rent.Apartment.Id && r.EndDate >= rent.RentStartDate)
                              .ToList()
                              .Any(r => r.IsConflicting(rent.RentStartDate, rent.RentEndDate)))
                ModelState.AddModelError("RentStartDate", "A megadott időpontban a szállás már foglalt! Kérem, válasszok más időpontot!");

            if (!ModelState.IsValid)
                return View("Index", rent);

            try
            {
                // ha minden megfelel, elmentjük az adatokat az adatbázisba
                Guest guest = new Guest
                {
                    UserName = "user" + Guid.NewGuid(), // a felhasználónevet generáljuk
                    Name = rent.GuestName,
                    Email = rent.GuestEmail,
                    Address = rent.GuestAddress,
                    PhoneNumber = rent.GuestPhoneNumber
                };
                _entities.Guest.Add(guest);

                _entities.Rent.Add(new Rent
                {
                    ApartmentId = rent.Apartment.Id,
                    UserName = guest.UserName,
                    StartDate = rent.RentStartDate,
                    EndDate = rent.RentEndDate
                });

                _entities.SaveChanges();

                // kiszámoljuk a teljes árat
                rent.TotalPrice = rent.Apartment.Price * Convert.ToInt32((rent.RentEndDate - rent.RentStartDate).TotalDays);

                ViewBag.Message = "A foglalását sikeresen rögzítettük!";
                return View("Result", rent);
            }
            catch
            {
                // ha valami hiba történt, visszairányítjuk a szerkesztőfelületre
                ModelState.AddModelError("", "A foglalás rögzítése sikertelen, kérem próbálja újra!");
                return View("Index", rent);
            }
        }        
    }
}